@@ -8,12 +8,12 @@
 #include "application.h"
 #include "audiosettings.h"
 #include "generalsettings.h"
+#include "global.h"
 #include "playbacksettings.h"
 #include "playlistitem.h"
+#include "subtitlessettings.h"
 #include "track.h"
 #include "tracksmodel.h"
-#include "global.h"
-#include "subtitlessettings.h"
 #include "videosettings.h"
 #include "worker.h"
 
@@ -29,36 +29,35 @@
 
 #include <KLocalizedString>
 #include <KShell>
+#include <QCommandLineParser>
 
 #if defined(Q_OS_UNIX)
-#include <QDBusConnection>
 #include "lockmanager.h"
 #include "mediaplayer2.h"
 #include "mediaplayer2player.h"
+#include <QDBusConnection>
 #endif
 
-MpvItem::MpvItem(QQuickItem * parent)
+MpvItem::MpvItem(QQuickItem *parent)
     : MpvCore(parent)
     , m_audioTracksModel(new TracksModel)
     , m_subtitleTracksModel(new TracksModel)
-    , m_playlistModel(new PlayListModel)
+    , m_playlistProxyModel(new PlayListProxyModel)
 {
-    mpv_observe_property(m_mpv, 0, "media-title",    MPV_FORMAT_STRING);
-    mpv_observe_property(m_mpv, 0, "time-pos",       MPV_FORMAT_DOUBLE);
+    m_playlistModel = new PlayListModel();
+    m_playlistProxyModel->setSourceModel(m_playlistModel);
+    mpv_observe_property(m_mpv, 0, "media-title", MPV_FORMAT_STRING);
+    mpv_observe_property(m_mpv, 0, "time-pos", MPV_FORMAT_DOUBLE);
     mpv_observe_property(m_mpv, 0, "time-remaining", MPV_FORMAT_DOUBLE);
-    mpv_observe_property(m_mpv, 0, "duration",       MPV_FORMAT_DOUBLE);
-    mpv_observe_property(m_mpv, 0, "pause",          MPV_FORMAT_FLAG);
-    mpv_observe_property(m_mpv, 0, "volume",         MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "mute",           MPV_FORMAT_FLAG);
-    mpv_observe_property(m_mpv, 0, "aid",            MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "sid",            MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "chapter",        MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "secondary-sid",  MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "contrast",       MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "brightness",     MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "gamma",          MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "saturation",     MPV_FORMAT_INT64);
-    mpv_observe_property(m_mpv, 0, "track-list",     MPV_FORMAT_NODE);
+    mpv_observe_property(m_mpv, 0, "duration", MPV_FORMAT_DOUBLE);
+    mpv_observe_property(m_mpv, 0, "pause", MPV_FORMAT_FLAG);
+    mpv_observe_property(m_mpv, 0, "volume", MPV_FORMAT_INT64);
+    mpv_observe_property(m_mpv, 0, "mute", MPV_FORMAT_FLAG);
+    mpv_observe_property(m_mpv, 0, "aid", MPV_FORMAT_INT64);
+    mpv_observe_property(m_mpv, 0, "sid", MPV_FORMAT_INT64);
+    mpv_observe_property(m_mpv, 0, "chapter", MPV_FORMAT_INT64);
+    mpv_observe_property(m_mpv, 0, "secondary-sid", MPV_FORMAT_INT64);
+    mpv_observe_property(m_mpv, 0, "track-list", MPV_FORMAT_NODE);
 
     initProperties();
 
@@ -85,12 +84,15 @@ MpvItem::MpvItem(QQuickItem * parent)
         }
     });
 
+    connect(m_playlistModel, &PlayListModel::playingItemChanged, this, [=]() {
+        command(QStringList() << "loadfile" << m_playlistModel->getPath());
+    });
+
 #if defined(Q_OS_UNIX)
     // register mpris dbus service
     QString mspris2Name(QStringLiteral("org.mpris.MediaPlayer2.haruna"));
     QDBusConnection::sessionBus().registerService(mspris2Name);
-    QDBusConnection::sessionBus().registerObject(QStringLiteral("/org/mpris/MediaPlayer2"),
-                                                 this, QDBusConnection::ExportAdaptors);
+    QDBusConnection::sessionBus().registerObject(QStringLiteral("/org/mpris/MediaPlayer2"), this, QDBusConnection::ExportAdaptors);
     // org.mpris.MediaPlayer2 mpris2 interface
     new MediaPlayer2(this);
     auto mp2Player = new MediaPlayer2Player(this);
@@ -115,7 +117,8 @@ MpvItem::MpvItem(QQuickItem * parent)
         Q_EMIT playPrevious();
     });
     connect(mp2Player, &MediaPlayer2Player::seek, this, [=](int offset) {
-        command(QStringList() << "add" << "time-pos" << QString::number(offset));
+        command(QStringList() << "add"
+                              << "time-pos" << QString::number(offset));
     });
     connect(mp2Player, &MediaPlayer2Player::openUri, this, [=](const QString &uri) {
         Q_EMIT openUri(uri);
@@ -147,7 +150,15 @@ void MpvItem::initProperties()
     setProperty("volume-max", "100");
     // set ytdl_path to yt-dlp or fallback to youtube-dl
     setProperty("script-opts", QString("ytdl_hook-ytdl_path=%1").arg(Application::youtubeDlExecutable()));
-    setProperty("ytdl-format", PlaybackSettings::ytdlFormat());
+
+    setProperty("sub-use-margins", SubtitlesSettings::allowOnBlackBorders() ? "yes" : "no");
+    setProperty("sub-ass-force-margins", SubtitlesSettings::allowOnBlackBorders() ? "yes" : "no");
+    QCommandLineParser *cmdParser = Application::instance()->parser();
+    QString ytdlFormat = PlaybackSettings::ytdlFormat();
+    if (cmdParser->isSet(QStringLiteral("ytdl-format-selection"))) {
+        ytdlFormat = cmdParser->value(QStringLiteral("ytdl-format-selection"));
+    }
+    setProperty("ytdl-format", ytdlFormat);
 
     setProperty("sub-font", SubtitlesSettings::fontFamily());
     setProperty("sub-font-size", SubtitlesSettings::fontSize());
@@ -162,6 +173,7 @@ void MpvItem::initProperties()
     setProperty("screenshot-template", VideoSettings::screenshotTemplate());
     setProperty("screenshot-format", VideoSettings::screenshotFormat());
 
+    setProperty("audio-client-name", "haruna");
     const QVariant preferredAudioTrack = AudioSettings::preferredTrack();
     setProperty("aid", preferredAudioTrack == 0 ? "auto" : preferredAudioTrack);
     setProperty("alang", AudioSettings::preferredLanguage());
@@ -182,31 +194,14 @@ void MpvItem::setPlaylistModel(PlayListModel *model)
     m_playlistModel = model;
 }
 
-QString MpvItem::playlistTitle()
+PlayListProxyModel *MpvItem::playlistProxyModel()
 {
-    return m_playlistTitle;
+    return m_playlistProxyModel;
 }
 
-void MpvItem::setPlaylistTitle(const QString &title)
+void MpvItem::setPlaylistProxyModel(PlayListProxyModel *model)
 {
-    if (title == playlistTitle()) {
-        return;
-    }
-    m_playlistTitle = title;
-    Q_EMIT positionChanged();
-}
-
-const QString &MpvItem::playlistUrl() const
-{
-    return m_playlistUrl;
-}
-
-void MpvItem::setPlaylistUrl(const QString &_playlistUrl)
-{
-    if (m_playlistUrl == _playlistUrl)
-        return;
-    m_playlistUrl = _playlistUrl;
-    Q_EMIT playlistUrlChanged();
+    m_playlistProxyModel = model;
 }
 
 QString MpvItem::mediaTitle()
@@ -335,62 +330,6 @@ void MpvItem::setSecondarySubtitleId(int value)
     Q_EMIT secondarySubtitleIdChanged();
 }
 
-int MpvItem::contrast()
-{
-    return getProperty("contrast").toInt();
-}
-
-void MpvItem::setContrast(int value)
-{
-    if (value == contrast()) {
-        return;
-    }
-    setProperty("contrast", value);
-    Q_EMIT contrastChanged();
-}
-
-int MpvItem::brightness()
-{
-    return getProperty("brightness").toInt();
-}
-
-void MpvItem::setBrightness(int value)
-{
-    if (value == brightness()) {
-        return;
-    }
-    setProperty("brightness", value);
-    Q_EMIT brightnessChanged();
-}
-
-int MpvItem::gamma()
-{
-    return getProperty("gamma").toInt();
-}
-
-void MpvItem::setGamma(int value)
-{
-    if (value == gamma()) {
-        return;
-    }
-    setProperty("gamma", value);
-    Q_EMIT gammaChanged();
-}
-
-int MpvItem::saturation()
-{
-    return getProperty("saturation").toInt();
-}
-
-void MpvItem::setSaturation(int value)
-{
-    if (value == saturation()) {
-        return;
-    }
-    setProperty("saturation", value);
-    Q_EMIT saturationChanged();
-}
-
 double MpvItem::watchPercentage()
 {
     return m_watchPercentage;
@@ -418,23 +357,18 @@ void MpvItem::setHWDecoding(bool value)
 {
     if (value) {
         setProperty("hwdec", "yes");
-    } else  {
+    } else {
         setProperty("hwdec", "no");
     }
     Q_EMIT hwDecodingChanged();
 }
 
-void MpvItem::loadFile(const QString &file, bool updateLastPlayedFile)
+void MpvItem::loadFile(const QString &file)
 {
     command(QStringList() << "loadfile" << file);
 
-    if (updateLastPlayedFile) {
-        GeneralSettings::setLastPlayedFile(file);
-        GeneralSettings::self()->save();
-    } else {
-        GeneralSettings::setLastPlaylistIndex(m_playlistModel->getPlayingVideo());
-        GeneralSettings::self()->save();
-    }
+    GeneralSettings::setLastPlayedFile(file);
+    GeneralSettings::self()->save();
 }
 
 void MpvItem::eventHandler()
@@ -454,16 +388,16 @@ void MpvItem::eventHandler()
             break;
         }
         case MPV_EVENT_END_FILE: {
-            auto prop = (mpv_event_end_file *)event->data;
+            auto prop = static_cast<mpv_event_end_file *>(event->data);
             if (prop->reason == MPV_END_FILE_REASON_EOF) {
                 Q_EMIT endFile("eof");
-            } else if(prop->reason == MPV_END_FILE_REASON_ERROR) {
+            } else if (prop->reason == MPV_END_FILE_REASON_ERROR) {
                 Q_EMIT endFile("error");
             }
             break;
         }
         case MPV_EVENT_PROPERTY_CHANGE: {
-            mpv_event_property *prop = (mpv_event_property *)event->data;
+            mpv_event_property *prop = static_cast<mpv_event_property *>(event->data);
 
             if (strcmp(prop->name, "time-pos") == 0) {
                 if (prop->format == MPV_FORMAT_DOUBLE) {
@@ -505,22 +439,6 @@ void MpvItem::eventHandler()
                 if (prop->format == MPV_FORMAT_INT64) {
                     Q_EMIT secondarySubtitleIdChanged();
                 }
-            } else if (strcmp(prop->name, "contrast") == 0) {
-                if (prop->format == MPV_FORMAT_INT64) {
-                    Q_EMIT contrastChanged();
-                }
-            } else if (strcmp(prop->name, "brightness") == 0) {
-                if (prop->format == MPV_FORMAT_INT64) {
-                    Q_EMIT brightnessChanged();
-                }
-            } else if (strcmp(prop->name, "gamma") == 0) {
-                if (prop->format == MPV_FORMAT_INT64) {
-                    Q_EMIT gammaChanged();
-                }
-            } else if (strcmp(prop->name, "saturation") == 0) {
-                if (prop->format == MPV_FORMAT_INT64) {
-                    Q_EMIT saturationChanged();
-                }
             } else if (strcmp(prop->name, "track-list") == 0) {
                 if (prop->format == MPV_FORMAT_NODE) {
                     loadTracks();
@@ -528,7 +446,7 @@ void MpvItem::eventHandler()
             }
             break;
         }
-        default: ;
+        default:;
             // Ignore uninteresting or unknown events.
         }
     }
@@ -548,39 +466,39 @@ void MpvItem::loadTracks()
     int subIndex = 1;
     int audioIndex = 0;
     for (const auto &track : tracks) {
-        const auto t = track.toMap();
-        if (track.toMap()["type"] == "sub") {
-            auto track = new Track();
-            track->setCodec(t["codec"].toString());
-            track->setType(t["type"].toString());
-            track->setDefaut(t["default"].toBool());
-            track->setDependent(t["dependent"].toBool());
-            track->setForced(t["forced"].toBool());
-            track->setId(t["id"].toLongLong());
-            track->setSrcId(t["src-id"].toLongLong());
-            track->setFfIndex(t["ff-index"].toLongLong());
-            track->setLang(t["lang"].toString());
-            track->setTitle(t["title"].toString());
-            track->setIndex(subIndex);
+        const auto trackMap = track.toMap();
+        if (trackMap["type"] == "sub") {
+            auto t = new Track();
+            t->setCodec(trackMap["codec"].toString());
+            t->setType(trackMap["type"].toString());
+            t->setDefaut(trackMap["default"].toBool());
+            t->setDependent(trackMap["dependent"].toBool());
+            t->setForced(trackMap["forced"].toBool());
+            t->setId(trackMap["id"].toLongLong());
+            t->setSrcId(trackMap["src-id"].toLongLong());
+            t->setFfIndex(trackMap["ff-index"].toLongLong());
+            t->setLang(trackMap["lang"].toString());
+            t->setTitle(trackMap["title"].toString());
+            t->setIndex(subIndex);
 
-            m_subtitleTracks.insert(subIndex, track);
+            m_subtitleTracks.insert(subIndex, t);
             subIndex++;
         }
-        if (track.toMap()["type"] == "audio") {
-            auto track = new Track();
-            track->setCodec(t["codec"].toString());
-            track->setType(t["type"].toString());
-            track->setDefaut(t["default"].toBool());
-            track->setDependent(t["dependent"].toBool());
-            track->setForced(t["forced"].toBool());
-            track->setId(t["id"].toLongLong());
-            track->setSrcId(t["src-id"].toLongLong());
-            track->setFfIndex(t["ff-index"].toLongLong());
-            track->setLang(t["lang"].toString());
-            track->setTitle(t["title"].toString());
-            track->setIndex(audioIndex);
+        if (trackMap["type"] == "audio") {
+            auto t = new Track();
+            t->setCodec(trackMap["codec"].toString());
+            t->setType(trackMap["type"].toString());
+            t->setDefaut(trackMap["default"].toBool());
+            t->setDependent(trackMap["dependent"].toBool());
+            t->setForced(trackMap["forced"].toBool());
+            t->setId(trackMap["id"].toLongLong());
+            t->setSrcId(trackMap["src-id"].toLongLong());
+            t->setFfIndex(trackMap["ff-index"].toLongLong());
+            t->setLang(trackMap["lang"].toString());
+            t->setTitle(trackMap["title"].toString());
+            t->setIndex(audioIndex);
 
-            m_audioTracks.insert(audioIndex, track);
+            m_audioTracks.insert(audioIndex, t);
             audioIndex++;
         }
     }
@@ -601,57 +519,6 @@ TracksModel *MpvItem::audioTracksModel() const
     return m_audioTracksModel;
 }
 
-void MpvItem::getYouTubePlaylist(const QString &path)
-{
-    m_playlistModel->clear();
-
-    // use youtube-dl to get the required playlist info as json
-    auto ytdlProcess = new QProcess();
-    ytdlProcess->setProgram(Application::youtubeDlExecutable());
-    ytdlProcess->setArguments(QStringList() << "-J" << "--flat-playlist" << path);
-    ytdlProcess->start();
-
-    QObject::connect(ytdlProcess, (void (QProcess::*)(int,QProcess::ExitStatus))&QProcess::finished,
-                     this, [=](int, QProcess::ExitStatus) {
-        // use the json to populate the playlist model
-        using Playlist = QList<PlayListItem*>;
-        Playlist m_playList;
-
-        QString json = ytdlProcess->readAllStandardOutput();
-        QJsonValue entries = QJsonDocument::fromJson(json.toUtf8())["entries"];
-        QString title = QJsonDocument::fromJson(json.toUtf8())["title"].toString();
-        if (entries.toArray().isEmpty()) {
-            Q_EMIT Global::instance()->error(i18n("Playlist is empty", title));
-            return;
-        }
-
-        setPlaylistTitle(title);
-        setPlaylistUrl(path);
-
-        QString playlistFileContent;
-
-        for (int i = 0; i < entries.toArray().size(); ++i) {
-            auto url = QString("https://youtu.be/%1").arg(entries[i]["id"].toString());
-            auto title = entries[i]["title"].toString();
-            auto duration = entries[i]["duration"].toDouble();
-
-            auto video = new PlayListItem(url, i, m_playlistModel);
-            video->setMediaTitle(!title.isEmpty() ? title : url);
-            video->setFileName(!title.isEmpty() ? title : url);
-
-            video->setDuration(Application::formatTime(duration));
-            m_playList.append(video);
-
-            playlistFileContent += QString("%1,%2,%3\n").arg(url, title, QString::number(duration));
-        }
-
-        // save playlist to disk
-        m_playlistModel->setPlayList(m_playList);
-
-        Q_EMIT youtubePlaylistLoaded();
-    });
-}
-
 void MpvItem::saveTimePosition()
 {
     // saving position is disabled
@@ -686,9 +553,9 @@ double MpvItem::loadTimePosition()
     auto hash = md5(getProperty("path").toString());
     auto configPath = Global::instance()->appConfigDirPath();
     KConfig *config = new KConfig(configPath.append("/watch-later/").append(hash));
-    int position = config->group("").readEntry("TimePosition", QString::number(0)).toDouble();
+    int pos = config->group("").readEntry("TimePosition", QString::number(0)).toDouble();
 
-    return position;
+    return pos;
 }
 
 void MpvItem::resetTimePosition()
@@ -700,6 +567,7 @@ void MpvItem::resetTimePosition()
     if (f.exists()) {
         f.remove();
     }
+    f.close();
 }
 
 void MpvItem::userCommand(const QString &commandString)
@@ -714,3 +582,5 @@ QString MpvItem::md5(const QString &str)
 
     return QString(md5.toHex());
 }
+
+#include "moc_mpvitem.cpp"
